-------------------------------------------------------------------------------
--  Copyright (c) 2018 Eclektek LLC. All rights reserved.
--
--
--  Developed by: William Carter
--
--  Permission is hereby granted, free of charge, to any person obtaining a copy of this software and  
--  associated documentation files (the "Software"), to deal with the Software without restriction,  
--  including without limitation the rights to use, copy, modify, merge, publish, sublicense and/or    
--  distribute copies of the Software, and to permit persons to whom the Software is furnished to do so,  
--  subject to the following conditions:
--
--      Redistributions of source code must retain the above copyright notice, this list of conditions 
--      and the following disclaimers.
--      Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
--      and the following disclaimers in the documentation and/or other materials provided with the distribution.
--      Neither the names of William Carter, Eclektek LLC, nor the names of its 
--      contributors may be used to endorse or promote products derived from this Software without 
--      specific prior written permission from Eclektek LLC.
--      This Software is provided free of charge and Eclektek LLC reserves the right to sell copies of the 
--      Software in its original form. Anyone sub-licensing this Software will provide it free of charge.
--
--  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
--  LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
--  IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
--  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
--  SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
-------------------------------------------------------------------------------
--  History :                                                                       
--                                                                                                                  
--  Date      By    Comments                                         
--  --------  ----  ------------------------------------------------------------
--  10/16/18  W.C.  Created
--
-------------------------------------------------------------------------------
--  Project       : ETHERNET_TESTER_PUBLIC                                                          
--  PCB           : any 
--  Device        : any
--  Entity Name   : ETH_DUT
--  File Name     : ETH_DUT.vhd
--  Design Parent : ETH_TESTER_V1                                                          
--                                                                             
-------------------------------------------------------------------------------
--  Dependencies:
--
--  ETH_DUT.vhd uses the following components:
--
--  ETH_DUT.vhd uses the following packages:
--    Package        none
--
-------------------------------------------------------------------------------
--  ETH_DUT.vhd
--
--  This is a first cut at an automated testbed for testing PHYNET node communication
--  error rates. Mainly, the difference between ETH_DUT and ETH_TESTER_V1 is
--  that ETH_DUT will support very large payload sizes and sete the DUT node
--  to DATA_EXCHANGE mode; limiting the payload_len for PHYNET DATA packets to 16384 bytes.
--
-------------------------------------------------------------------------------
--  Limitations
--
------------------------------------------------------------------------------
--  ToDo
--
--
-------------------------------------------------------------------------------
--  NOTES:
--
-------------------------------------------------------------------------------



Library ieee;
  use ieee.std_logic_1164.all;
--  use ieee.numeric_std.all;
  use ieee.std_logic_arith.all;
  use ieee.std_logic_unsigned.all;
  
entity ETH_DUT is
  port(
        -- GLOBAL SIGNALS
        clk100                : in    std_logic; -- 100 MHz clock for controls
        clk125                : in    std_logic; -- 125 MHz clock for Ethernet transmitter clock domain
        rxclk                 : in    std_logic; -- 125 MHz clock for Ethernet receiver clock domain 
        areset                : in    std_logic; -- external asynchronous reset
        
        phyrxer               : in    std_logic;
        phy_rxdv              : in    std_logic;
        phy_rxd               : in    std_logic_vector(7 downto 0);
        phy_txen              : out   std_logic;
        phy_txd               : out   std_logic_vector(7 downto 0);
        mdclk_out             : out   std_logic;
        mdio_in               : in    std_logic;
        mdio_trst_en          : out   std_logic;
                        
        -- TTL USB UART
        UartRx                : in    std_logic; -- Adafruit TTL UART Green wire
        UartTx                : out   std_logic   -- Adafruit TTL UART White wire 
  );
end entity;

architecture rtl of ETH_DUT is

-------------------------------------------------------------------------------
--  Component Declarations
-------------------------------------------------------------------------------

COMPONENT FLOWTHRU_4kX8
  PORT (
    rst         : IN STD_LOGIC;
    wr_clk      : IN STD_LOGIC;
    rd_clk      : IN STD_LOGIC;
    din         : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    wr_en       : IN STD_LOGIC;
    rd_en       : IN STD_LOGIC;
    dout        : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    full        : OUT STD_LOGIC;
    empty       : OUT STD_LOGIC;
    prog_empty  : OUT STD_LOGIC);
END COMPONENT;

COMPONENT ila_0

PORT (
  clk     : IN STD_LOGIC;
  probe0  : IN STD_LOGIC_VECTOR(7 DOWNTO 0); 
  probe1  : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
  probe2  : IN STD_LOGIC_VECTOR(7 DOWNTO 0)
);
END COMPONENT  ;

-------------------------------------------------------------------------------
--  Type Declarations
-------------------------------------------------------------------------------

type tx_states is  (TX_IDLE,    -- run_b = '0'
                    TX_START,
                    TX_PREAMBLE,
                    TX_DATA,
                    TX_FIFORST
                    );       

type rx_states is  (RX_IDLE,    -- run_b = '0'
                    RX_WAIT_DV, 
                    RX_WAIT_SDF,
                    RX_DATA,
                    RX_BAD_PREAMBLE
                    );       
                    
                    
-------------------------------------------------------------------------------
--  Constant Declarations
-------------------------------------------------------------------------------

  constant PREAMBLE_SYMBOLS               : std_logic_vector(63 downto 0) := X"55555555555555D5";
  constant KC705_ETHERNET_PHY_ADDRESS     : std_logic_vector(4 downto 0)  := "00111";  -- from schematic
  
-------------------------------------------------------------------------------
--  Signal Declarations
-------------------------------------------------------------------------------

signal  current_txs, next_txs        : tx_states; 
signal  current_rxs, next_rxs        : rx_states; 

-- User Interface Registers
signal  control_r             :  std_logic_vector(47 downto 0); -- wr to logic      control the test
signal  rxerror_count_r       :  std_logic_vector(47 downto 0); -- rd from logic    report the total number of non-matching payload bytes 

-- control bits
signal  soft_reset_b          : std_logic;

signal  t_reset               : std_logic;

-- Ethernet Transmit clock domain

signal  phy_tx_preamble           : std_logic_vector(7 downto 0);
signal  tx_preamble_sr            : std_logic_vector(63 downto 0);
signal  rx_preamble_sr            : std_logic_vector(63 downto 0);

-- MD I2C interface
signal  clk2_5mhz             : std_logic;
signal  clk2_5mhz_shift       : std_logic;
signal  mdio_zen              : std_logic;
signal  phy_reg_dout          : std_logic_vector(15 downto 0); 
signal  phy_reg_din           : std_logic_vector(15 downto 0); 
signal  phy_reg_addr          : std_logic_vector(4 downto 0);
signal  mdi_reg_rden          : std_logic;
signal  mdi_reg_wren          : std_logic;
signal  mdi_rd_ack            : std_logic;
signal  mdi_wr_ack            : std_logic;
signal  mdi_access_rddata_r      : std_logic_vector(47 downto 0);
signal  mdi_access_ctrl_r       : std_logic_vector(47 downto 0);
signal  rxd_fifo_data         : std_logic_vector(7 downto 0); 
signal  rxd_fifo_wrfull       : std_logic; 
signal  rxd_fifo_rdempty      : std_logic; 
signal  rxd_fifo_rdvalid      : std_logic;
signal  rxd_fifo_rden         : std_logic;
signal  rxd_fifo_wren         : std_logic;
signal  rxd_fifo_reset        : std_logic;
signal  rxd_almost_empty      : std_logic;
signal  rxerr_dcount          : std_logic_vector(9 downto 0);
signal  phyrxerr_r            : std_logic;
signal  rxerr_ledge           : std_logic;
signal  rxerr_fedge           : std_logic;
signal  reg_13_rd_stb         : std_logic;
signal  test_fifo_reset       : std_logic;
signal  send_packet_stb       : std_logic;
signal  kill_tx               : std_logic;
signal ireg_address       : std_logic_vector(3 downto 0);
signal ireg_data_out      : std_logic_vector(47 downto 0);
signal ireg_write_stb     : std_logic;
signal ireg_data_in       : std_logic_vector(47 downto 0);
signal ireg_read_stb      : std_logic;
signal ibusy              : std_logic;
signal dbg_reg0_r         : std_logic_vector(47 downto 0); 
signal dbg_reg1_r         : std_logic_vector(47 downto 0); 
signal dbg_reg2_r         : std_logic_vector(47 downto 0); 
signal control_reg        : std_logic_vector(47 downto 0);
signal itxen              : std_logic;
signal itxd               : std_logic_vector(7 downto 0); 
signal rx_payload_count   : std_logic_vector(31 downto 0);

signal  dbg_rx_state            : std_logic_vector(3 downto 0);
signal  dbg_tx_state            : std_logic_vector(3 downto 0);

signal  icon_ctrl             : std_logic_vector(35 downto 0);
signal  probe0                : std_logic_vector(7 downto 0);    
signal  probe1                : std_logic_vector(7 downto 0);    
signal  probe2                : std_logic_vector(7 downto 0);
signal  probe3                : std_logic_vector(7 downto 0);

                                         
-------------------------------------------------------------------------------
--  Procedure Declarations
-------------------------------------------------------------------------------
  
  
begin


-------------------------------------------------------------------------------
--  Component Instantiations
-------------------------------------------------------------------------------

  RXD_DATA_FIFO : FLOWTHRU_4kX8
  PORT MAP (
    rst         => rxd_fifo_reset,
    wr_clk      => rxclk,
    rd_clk      => clk125,
    din         => phy_rxd,
    wr_en       => rxd_fifo_wren,
    rd_en       => rxd_fifo_rden,
    dout        => rxd_fifo_data,
    full        => rxd_fifo_wrfull,
    empty       => rxd_fifo_rdempty,
    prog_empty  => rxd_almost_empty
  );
  
                  

  ILA0_inst : ila_0
  PORT MAP (
    clk => clk125,
    probe0 => probe0, 
    probe1 => probe1,
    probe2 => probe2
  );

    probe0    <= rxd_fifo_data;   
    probe1    <= itxd;                                                  
    probe2    <= '0' &   -- b7                      
                 phy_rxdv &  -- b6                         
                 itxen &     -- b5       
                 '0' &       -- b4          
                 '0' &       -- b3
                 rxd_fifo_rdvalid &       -- b2
                 rxd_fifo_rdempty &       -- b1
                 '0';        -- b0
                               
-------------------------------------------------------------------------------
--  Concurrent Assignments
-------------------------------------------------------------------------------

  -- Control Register bit assignments
--  mdi_ctr_rden_b    <= control_r(17);
--  mdi_ctr_wren_b    <= control_r(16);
  
 
                                            
                        
  -- Resets
  -- everything gets reset from RESET button or soft_reset_b
  -- measurement counters and accumulators have individual reset control bits
  t_reset           <= '1' when areset = '1' or soft_reset_b = '1' else '0';
  
  
  mdi_access_rddata_r    <= X"00000000" & phy_reg_dout;
  phy_reg_addr           <= mdi_access_ctrl_r(28 downto 24);
  phy_reg_din            <= mdi_access_ctrl_r(15 downto 0);
  
--type rx_states is  (RX_IDLE,    -- run_b = '0'
--                    RX_WAIT_DV, 
--                    RX_WAIT_SDF,
--                    RX_DATA
--                    );       

  dbg_rx_state  <=  "1111" when current_rxs = RX_IDLE else
                    "0001" when current_rxs = RX_WAIT_DV else
                    "0010" when current_rxs = RX_WAIT_SDF else
                    "0011" when current_rxs = RX_DATA else
                    "0100" when current_rxs = RX_BAD_PREAMBLE else
                    "0000";
                  
--type tx_states is  (TX_IDLE,    -- run_b = '0'
--                    TX_START,
--                    TX_PREAMBLE,
--                    TX_DATA,
--                    TX_FIFORST
--                    );       
  dbg_tx_state  <=  "1111" when current_txs = TX_IDLE else
                    "0001" when current_txs = TX_START else
                    "0010" when current_txs = TX_PREAMBLE else
                    "0011" when current_txs = TX_DATA else
                    "0100" when current_txs = TX_FIFORST else
                    "0000";
  
-------------------------------------------------------------------------------
--  Process Declarations
-------------------------------------------------------------------------------
 

  phy_tx_preamble   <= tx_preamble_sr(63 downto 56);
    
  itxen    <= '1' when current_txs = TX_PREAMBLE else
                 '1' when current_txs = TX_DATA else
                 '0';
                 
  itxd     <= phy_tx_preamble when current_txs = TX_PREAMBLE else
                 rxd_fifo_data when current_txs = TX_DATA else
                 X"FF";
    
  send_packet_stb <= '1' when next_rxs = RX_DATA and rxd_almost_empty = '0' else '0'; 

 
-- Ethernet Transmit clock domain

  TX_PREAMBLE_PROC : process(clk125, t_reset)
  begin
    if t_reset = '1' then
      tx_preamble_sr   <= PREAMBLE_SYMBOLS;
    elsif rising_edge(clk125) then
      if current_txs = TX_FIFORST then
        tx_preamble_sr   <= PREAMBLE_SYMBOLS;
      elsif  current_txs = TX_IDLE then
        tx_preamble_sr   <= PREAMBLE_SYMBOLS;
      elsif  current_txs = TX_PREAMBLE then
        tx_preamble_sr   <= tx_preamble_sr(55 downto 0) & X"00";
      end if;  
    end if;
  end process TX_PREAMBLE_PROC;
        
  
  kill_tx   <= '1' when phyrxer = '1' else
               '1' when soft_reset_b = '1' else
               '0';

--  kill_tx   <= '0';
               
                   
-- a two-processs state machine for the transmitter
  -- the clocked portion
  TX_UPDATE_STATE : process(clk125, areset)
  begin
    if areset = '1' then
      current_txs   <= TX_IDLE;
    elsif rising_edge(clk125) then
      if kill_tx = '1' then
        current_txs   <= TX_FIFORST;
      else  
        current_txs   <= next_txs;
      end if;  
    end if;
  end process TX_UPDATE_STATE;
 
    
--type tx_states is  (TX_IDLE,    -- run_b = '0'
--                    TX_START,
--                    TX_PREAMBLE,
--                    TX_DATA,
--                    TX_FIFORST
--                    );       

  -- the state machine
  TX_PROCESS_SM : process (current_txs, phy_tx_preamble, send_packet_stb, rxd_almost_empty)
  begin
    case current_txs is
      when TX_IDLE =>
        if send_packet_stb = '1' then
           next_txs  <= TX_START;
        else
           next_txs  <= TX_IDLE;
        end if; 
          
      when TX_START =>
        next_txs  <= TX_PREAMBLE;
                
      -- Send the preamble  
      when TX_PREAMBLE =>
        if phy_tx_preamble = X"D5" then
           next_txs  <= TX_DATA;
        else
           next_txs  <= TX_PREAMBLE;
        end if;   
               
      -- send the payload data   
      when TX_DATA =>
        if rxd_almost_empty = '1' then
           next_txs  <= TX_FIFORST; 
        else
           next_txs  <= TX_DATA;
        end if;   
              
      -- Always reset the TxD payload capture FIFO before each packet            
      when TX_FIFORST =>
        next_txs  <= TX_IDLE;
                
      when others =>
        next_txs  <= TX_IDLE;
      
    end case;    
  end process TX_PROCESS_SM; 

                      
  rxd_fifo_rden   <= '1' when current_txs = TX_DATA else
--                     '1' when rxd_fifo_rdempty = '0' else
                     '0'; 

-- Ethernet Receive clock domain


  rxd_fifo_wren     <= '1' when current_rxs = RX_DATA else '0';
  
  rxd_fifo_reset  <= '1' when t_reset = '1' else 
                     '1' when phyrxer = '1' else
                     '1' when current_txs = TX_FIFORST else
                     '0';


  RX_PREAMBLE_PROC : process(rxclk, t_reset)
  begin
    if t_reset = '1' then
      rx_preamble_sr   <= (others => '0');
    elsif rising_edge(rxclk) then
      if current_rxs = RX_WAIT_DV then
        rx_preamble_sr   <= rx_preamble_sr(55 downto 0) & phy_rxd;
      elsif current_rxs = RX_WAIT_SDF then
        rx_preamble_sr   <= rx_preamble_sr(55 downto 0) & phy_rxd;
      end if;  
    end if;
  end process RX_PREAMBLE_PROC;
  
  process(rxclk, t_reset)
  begin
    if t_reset = '1' then
      phyrxerr_r   <= '0';
    elsif rising_edge(rxclk) then
      phyrxerr_r   <= phyrxer;
    end if;
  end process;
  
  process(rxclk, t_reset)
  begin
    if t_reset = '1' then
      rxerr_ledge   <= '0';
    elsif rising_edge(rxclk) then
      if phyrxer = '1' and phyrxerr_r = '0' then
        rxerr_ledge   <= '1';
      else
        rxerr_ledge   <= '0';
      end if;
    end if;
  end process;
   
  process(rxclk, t_reset)
  begin
    if t_reset = '1' then
      rxerr_fedge   <= '0';
    elsif rising_edge(rxclk) then
      if phyrxer = '0' and phyrxerr_r = '1' then
        rxerr_fedge   <= '1';
      else
        rxerr_fedge   <= '0';
      end if;
    end if;
  end process;
  
  
  -- count all the times the PHY throws an rxerr
  RXERR_COUNT_PROC : process(rxclk, t_reset)
  begin
    if t_reset = '1' then
      rxerror_count_r   <= (others => '0');
    elsif rising_edge(rxclk) then
      if rxerr_fedge = '1' then
        rxerror_count_r   <= rxerror_count_r + 1;
      else  
        rxerror_count_r   <= rxerror_count_r;
      end if;  
    end if;
  end process RXERR_COUNT_PROC;
  
  
-- a two-processs state machine for the recviever
  -- the clocked portion
  RX_UPDATE_STATE : process(rxclk, t_reset)
  begin
    if t_reset = '1' then
      current_rxs   <= RX_IDLE;
    elsif rising_edge(rxclk) then
      if kill_tx = '1' then
        current_rxs   <= RX_IDLE;
      else  
        current_rxs   <= next_rxs;
      end if;  
    end if;
  end process RX_UPDATE_STATE;
 
    
--type rx_states is  (RX_IDLE,    -- run_b = '0'
--                    RX_WAIT_DV, 
--                    RX_WAIT_SDF,
--                    RX_DATA,
--                    RX_BAD_PREAMBLE,
--                    );       

  -- the state machine
  RX_PROCESS_SM : process (current_rxs, phy_rxdv, phyrxer, phy_rxd)
  begin
    case current_rxs is
      when RX_IDLE =>
        if phy_rxdv = '1' then
           next_rxs  <= RX_WAIT_DV;
        else
           next_rxs  <= RX_IDLE;
        end if; 
        
      -- wait for valid RxD data    
      when RX_WAIT_DV =>
        if phy_rxdv = '0' then
           next_rxs  <= RX_IDLE;
        elsif phy_rxdv = '1' and phy_rxd = X"55" then
           next_rxs  <= RX_WAIT_SDF;
        else
           next_rxs  <= RX_BAD_PREAMBLE;
        end if; 
        
      -- look for the preamble SFD symbol  
      when RX_WAIT_SDF =>
        if phy_rxdv = '1' and phy_rxd = X"D5" then
           next_rxs  <= RX_DATA;
        elsif phy_rxdv = '1' and phy_rxd = X"55" then
           next_rxs  <= RX_WAIT_SDF;
        else
           next_rxs  <= RX_BAD_PREAMBLE;
        end if; 
      
      when RX_DATA =>
        if phy_rxdv = '0' then
          next_rxs  <= RX_IDLE;
        else
          next_rxs  <= RX_DATA;
        end if;  
                
      when RX_BAD_PREAMBLE =>
        if phy_rxdv = '0' then
          next_rxs  <= RX_IDLE;
        else
          next_rxs  <= RX_BAD_PREAMBLE;
        end if;   
                
      when others =>
        next_rxs  <= RX_IDLE;
      
    end case;    
  end process RX_PROCESS_SM; 
  
  
  
  -- for this application the bottom 8 registers are read/write from the UART
  DBG_REG0_PROC : process(clk100, areset)
  begin
    if areset = '1' then
      dbg_reg0_r   <= (others => '0');
    elsif rising_edge(clk100) then
      if (ireg_write_stb = '1' and ireg_address = X"0") then
          dbg_reg0_r   <= ireg_data_out;
      end if;
    end if;
  end process DBG_REG0_PROC;

  DBG_REG1_PROC : process(clk100, areset)
  begin
    if areset = '1' then
      dbg_reg1_r   <= (others => '0');
    elsif rising_edge(clk100) then
      if (ireg_read_stb = '1' and ireg_address = X"1") then
        dbg_reg1_r   <= rxerror_count_r;
      end if;
    end if;
  end process DBG_REG1_PROC;
  
-- user interface  
  ireg_data_in  <= dbg_reg0_r when ireg_address = X"0" else
                   dbg_reg1_r when ireg_address = X"1" else
                   dbg_reg2_r when ireg_address = X"2" else
                   X"000000000000";
                   
  dbg_reg2_r    <=  rx_preamble_sr(47 downto 0);   
                                   
  control_reg   <=  dbg_reg0_r;
  soft_reset_b  <=  control_reg(7);
    
-------------------------------------------------------------------------------
--  Other Output Assignments
-------------------------------------------------------------------------------
    
  phy_txen    <= itxen;
                 
  phy_txd     <= itxd;
                 
  mdclk_out       <= '1';
  mdio_trst_en    <= '0';           
                 

end rtl;
