#-----------------------------------------------------------------------------
#  Copyright (c) 2018 Eclektek LLC. All rights reserved.
#
#
#  Developed by: William Carter
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy of this software and  
#  associated documentation files (the "Software"), to deal with the Software without restriction,  
#  including without limitation the rights to use, copy, modify, merge, publish, sublicense and/or    
#  distribute copies of the Software, and to permit persons to whom the Software is furnished to do so,  
#  subject to the following conditions:
#
#      Redistributions of source code must retain the above copyright notice, this list of conditions 
#      and the following disclaimers.
#      Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
#      and the following disclaimers in the documentation and/or other materials provided with the distribution.
#      Neither the names of William Carter, Eclektek LLC, nor the names of its 
#      contributors may be used to endorse or promote products derived from this Software without 
#      specific prior written permission from Eclektek LLC.
#      This Software is provided free of charge and Eclektek LLC reserves the right to sell copies of the 
#      Software in it original form. Anyone sub-licensing this Software will provide it free of charge.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
#  LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
#  IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
#  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
#  SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
#-----------------------------------------------------------------------------
#  History :                                                                       
#                                                                                                                  
#  Date      By    Version  Comments                                         
#  --------  ----  -------  -----------------------------------------------------
#  09/19/18  W.C.  1.00     Created
#
#-----------------------------------------------------------------------------
# Tester.py was written using Python 2.7 on Windows 7
# Tester.py requires the following non-standard Library modules:
#   pyserial-2.6
#   
#-----------------------------------------------------------------------------
# Tester.py communicates with the ATLYS ATLYS_ETH_TESTER
# to setup, execute and acquire test PHY Loopback results using a USB UART
#
# Tester only functions with the ATLYS ATLYS_ETH_TESTER bit file.
#
# Command line Useage: <Python Path>python Tester.py [-p port] [-l off]
#  where <Python Path> is the location of your Python version 2.7 or less installation
#  [port] is the name of the USB UART port connected to your FPGA board UART connector
#  [off] disables data logging ( anything else or absense of the -l options enables data logging
#
#   Windows example:  C:\Python27\python27 Tester.py -p COM9 -l off
#   Linux example:    python Tester.py -p /dev/ttyUSB0 
#
# Tester first checks that the specified UART port is available. If not it exits. Tester then
# tries to write a value to UART_RegAccess Register 0 and read it back. If unsuccessful it exits.
# If the UART_RegAccess test succeeds a menu is displayed in the terminal window or DOS box.
# Tester Commands are:
#   'Q' - Quit Tester
#   'D' - Set Inter-Packet Delay in 125MHz clock periods (0x7d - 0xFFFFFFFF)
#   'P' - Set Packet Payload length in bytes (0x10 - 0xFFFFFFFF)
#   'S' - Set Injected Error Rate (0, 1/6, 2/6, 3/6, 4/6, 5/6, 6/6)
#   'M' - Read PHY Register ( 0 to 31 )
#   'R' - Run a Test
#   'Z' - Print out the Current Tester User Interface Registers
#   'N' - Set a Limit for how many Packets to send in a Test Run
#
#
# Tester writes a uniquely timestamped log file, unless disabled by the '-l off' command line option, in the directory
# where Tester.py is located. A new file is created each time that Tester is run.
#
# 
#-----------------------------------------------------------------------------
# NOTES:
#
#   - On some FPGA boards the UART is powered by the USB hub or PC and will be enumerated and available whenever a
#     cable connects the FPGA UART connector to a PC or PC connected hub, even if the FPGA board power switch is off.
#   - in Windows 7 the appropriate UART port can be found using the Windows Device Manager and is typically COMxx. In
#     Linux typing 'dmesg | grep tty' in a terminal will display all of the UARTS that are available. In either OS
#     comparing what's available when the FPGA UART is not attached to what's available when it is attached helps 
#     identify the correct port in case of confusion.
# 
#-----------------------------------------------------------------------------
# ToDo:
#
#
#-----------------------------------------------------------------------------
# KNOWN ISSUES:
#
#
#-----------------------------------------------------------------------------

# Modules
import serial
import time
import math
import sys

# Some definitions

# FPGA UART_RegAccess Register Assignments

test_control_reg          = '0'
test_interval_reg         = '1'
test_reg_2_reg            = '2'
payload_config_reg        = '3'
mdi_access_ctrl_reg       = '4'
mdi_access_rddata_reg     = '5'
test_phy_rxerr_reg        = '6'
test_readback_reg         = '7'
test_status_reg           = '8'
test_txpckt_count_reg     = '9'
test_error_count_reg      = 'A'
test_rxbyte_count_reg     = 'B'
test_elapsed_time_reg     = 'C'
test_rxerr_fifo_data_reg  = 'D' 
test_txbyte_count_reg     = 'E' 
test_rxpckt_count_reg     = 'F' 

regd = {'0': 'test_control_reg          ',
        '1': 'test_interval_reg         ',
        '3': 'payload_config_reg        ',
        '4': 'mdi_access_ctrl_reg       ',
        '5': 'mdi_access_rddata_reg     ',
        '6': 'test_phy_rxerr_reg        ',
        '8': 'test_status_reg           ',
        '9': 'test_txpckt_count_reg     ',
        'A': 'test_error_count_reg      ',
        'B': 'test_rxbyte_count_reg     ',
        'C': 'test_elapsed_time_reg     ',
        'D': 'test_rxerr_fifo_data_reg  ',
        'E': 'test_txbyte_count_reg     ',
        'F': 'test_rxpckt_count_reg     '}

# Control bit field assignments
## Reg0
#  mdi_ctr_rden_b    <= control_r(17);
#  byte_ctr_rst_b    <= control_r(9);                              
#  error_ctr_rst_b   <= control_r(8);                              
#  soft_reset_b      <= control_r(7);
#  packet_mode_b     <= control_r(1);                              
#  run_b             <= control_r(0);
byte_ctr_reset_mask       = 0x000000000200
error_ctr_reset_mask      = 0x000000000100
soft_reset_mask           = 0x000000000080
run_n_packet_mode_mask    = 0x000000000010
run_mask                  = 0x000000000001

# payload_config_reg bit field assignments
payload_length_mask       = 0x0000FFFFFFFF
max_packet_limit_mask     = 0xFFFF00000000

# Status bit field assignments
## Reg8
#  status_r          <= X"000" & '0' &           -- b47..b35 unassigned
#                       rxerr_dcount &           -- b34..b24
#                       dbg_tx_state &           -- b23..b20
#                       dbg_rx_state &           -- b19..b16
#                       X"00" &                  -- b15..b8 unassigned
#                       "0000000" &              -- b7..b1 unassigned
#                       test_is_running;         -- b0
 
err_fifo_count_mask       = 0x000FFF000000  
tx_state_mask             = 0x000000F00000                    
rx_state_mask             = 0x0000000F0000                    
test_is_running_mask      = 0x000000000001 

statd = {0x000000000001: 'test_is_running   '}



#Reset values in hex
set_mdi_rden            = '20000'
set_mdi_wren            = '10000'
set_byte_cnt_reset      = '200'                              
sete_error_cnt_reset    = '100'                               
set_soft_reset          = '80'  
set_error_cnt_reset     = '100'
set_byte_ctr_reset      = '200'


# NOTE: in some cases such as R4, R5, R6 and R7 unused bits are 0
#       so the mask values is mainly provided for informational purposes

# Reg15 FPGA I2C Master interface command set
two_twenty_eight_pwr    = 268435456
two_thirty_seven_pwr    = 137438953472
two_thirty_eight_pwr    = 274877906944
two_fourty_five_pwr     = 35184372088832
two_fourty_four_pwr     = 17592186044416
two_to_eight_pwr        = 256
two_to_sixteen_pwr      = 65536 
two_to_twenty_four_pwr  = 16777216 
two_to_thirty_two_pwr   = 4294967296 


# command line options
uart_port = 'COM5'
save_log = 1

max_tx_packets  = 0
mode = 0
packet_length = 16

# Functions

def get_results():
  t = [0,0]
  TestData = 'R 4\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  x = int(ResponseData[4:18],16)
  t[0] = x
  TestData = 'R 5\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  x = int(ResponseData[4:18],16)
  t[1] = x
  return t
  
  

      
# get command-line options
def get_options(argv):
  clo= {}
  while argv:
    if argv[0][0] == '-':
      clo[argv[0]] = argv[1]
      argv = argv[2:]
    else:
      argv = argv[1:]
  return clo

# function to write payload length to  payload_config_reg
def write_payload_config():
  global max_tx_packets
  global packet_length
  n = max_tx_packets * 4294967296
  c = raw_input('\nEnter Payload Length in Hex => ')
  s0 = int(c,16)
  if s0 > 42949672955:
    s0 = 42949672955
  packet_length = s0  
  s0 = s0 + n  
  s = format(s0,'012x')
  TestData = 'W' + ' ' + payload_config_reg + ' ' + s.upper() + '\x0D\x0A'  
  ser.write(TestData)
  
       

# function to status registers
def read_status():
  TestData = 'R ' + test_status_reg + '\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
#  print 'Reg8 = ', ResponseData[4:18]
#  if save_log == 1:
#    print >> log,'Reg8 = ', ResponseData[4:18]
  return int(ResponseData[4:18], 16) 
  

      
# function to print all registers
# '0': 'test_control_reg          ',
# '1': 'test_interval_reg         ',
# '3': 'payload_config_reg        ',
# '4': 'mdi_access_ctrl_reg       ',
# '5': 'mdi_access_rddata_reg     ',
# '6': 'test_phy_rxerr_reg        ',
# '8': 'test_status_reg           ',
# '9': 'test_txpckt_count_reg     ',
# 'A': 'test_error_count_reg      ',
# 'B': 'test_rxbyte_count_reg     ',
# 'C': 'test_elapsed_time_reg     ',
# 'D': 'test_rxerr_fifo_data_reg  ',
# 'E': 'test_txbyte_count_reg     ',
# 'F': 'test_rxpckt_count_reg     '}

def show_all():
  e = ['0', '1', '3', '6', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F']
  for x in e:
    TestData = 'R ' + x + '\x0D\x0A'
    ser.write(TestData)
    ResponseData = ser.read(19)
    print regd[x] + ' = ', ResponseData[4:18]
    if save_log == 1:
      print >> log,regd[x] + ' = ', ResponseData[4:18]

# Function to print out test statistics
def show_results():
#statd = {0x000000000001: 'test_is_running   '}
  e = [0x000000000001]

  #show status
  TestData = 'R ' + test_status_reg + '\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  print 'Status  = ', ResponseData
  if save_log == 1:
    print >> log, 'Status  = ', ResponseData
  n = int(ResponseData[4:18], 16)
  for x in e:
    if n & x != 0:
      print statd[x], ' is set'
      if save_log == 1:
        print >> log, statd[x], ' is set'              

  #show payload size
  TestData = 'R ' + payload_config_reg + '\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  l = int(ResponseData[4:18], 16)
  l = l & 0xFFFFFFFF
  if l == 0:
    l = 16  
  print 'Payload size = ', l
  if save_log == 1:
    print >> log, 'Payload Size = ', l

  #show TX packet count
  TestData = 'R ' + test_txpckt_count_reg + '\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  p = int(ResponseData[4:18], 16)
  print 'Total Number of TEST Packets sent = ', p
  if save_log == 1:
    print >> log, 'Total Number of TEST Packets sent = ', p

  #show RX packet count
  TestData = 'R ' + test_rxpckt_count_reg + '\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  p = int(ResponseData[4:18], 16)
  print 'Total Number of TEST Packets received = ', p
  if save_log == 1:
    print >> log, 'Total Number of TEST Packets received = ', p

  #show TX byte count
  TestData = 'R ' + test_txbyte_count_reg + '\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  n = int(ResponseData[4:18], 16)
  print 'Total Number of TEST Bytes sent = ', n
  if save_log == 1:
    print >> log, 'Total Number of TEST Bytes sent = ', n

  #show RX byte count
  TestData = 'R ' + test_rxbyte_count_reg + '\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  r = int(ResponseData[4:18], 16)
  print 'Total Number of TEST Bytes received = ', r
  if save_log == 1:
    print >> log, 'Total Number of TEST Bytes received = ', r

  #show error count
  TestData = 'R ' + test_error_count_reg + '\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  e = int(ResponseData[4:18], 16)
  print 'Total Number of Errors = ', e
  if save_log == 1:
    print >> log, 'Total Number of Errors = ', e
    
  #show missing packet count    
  TestData = 'R ' + test_phy_rxerr_reg     + '\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  k = int(ResponseData[4:18], 16)
  print 'Total Number of PHY rxerr Events = ', k
  if save_log == 1:
    print >> log, 'Total Number of PHY rxerr Events = ', k

  #show elapsed time
  TestData = 'R ' + test_elapsed_time_reg + '\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  v = int(ResponseData[4:18], 16)
  t = float(v)/125000000.0
  print 'Total Duration of the test (in Seconds) = ', t
  if save_log == 1:
    print >> log, 'Total Duration of the test (in Seconds) = ', t
  
  if n > 0:  
    x = 100.0*float(e)/float(n)
    print 'Percentage Errors = ', x
    if save_log == 1:
      print >> log, 'Percentage Errors = ', x
  
  x = float(p)/t
  print 'TEST packets sent per second = ', x
  if save_log == 1:
    print >> log, 'TEST packets sent per second = ', x
  
  x = float(n)/t
  print 'Tx Data Rate (Bytes/s) = ', x
  if save_log == 1:
    print >> log, 'Tx Data Rate (Bytes/s) = ', x
  
  x = float(r)/t
  print 'Rx Data Rate (Bytes/s) = ', x
  if save_log == 1:
    print >> log, 'Rx Data Rate (Bytes/s) = ', x
  
  x = float(k)/t
  print 'PHY rxerr Events per second = ', x
  if save_log == 1:
    print >> log, 'PHY rxerr Events per second = ', x
  
  x = 100.0*float(k)/float(p)
  print 'Percentage of packets with PHY rxerr Events = ', x
  if save_log == 1:
    print >> log, 'Percentage of packets with PHY rxerr Events = ', x
    
      
          

# function to force the state machine into IDLE
def reset_test():
  TestData = 'W ' + test_control_reg + ' ' + set_soft_reset + '\x0D\x0A'  # 
  ser.write(TestData)
  time.sleep(.2)              # delay 
  TestData = 'W ' + test_control_reg + ' ' + '0\x0D\x0A'  # 
  time.sleep(.2)              # delay 
 
  
# function to read a Ethernet PHY register                 
# cstr is a string with the register address             
def read_phy_reg(cstr):
  # write the resister address offset by 24 bits
  TestData = 'W ' + mdi_access_ctrl_reg + ' ' + cstr + '000000' + '\x0D\x0A'
  ser.write(TestData)
  
  # Strobe the MDI read enable control bit
  TestData = 'W ' + test_control_reg + ' ' + '20000' + '\x0D\x0A' 
  ser.write(TestData)
  time.sleep(.1)              # delay 
  TestData = 'W ' + test_control_reg + ' ' + '0\x0D\x0A' 
  ser.write(TestData)
  time.sleep(.2)              # delay 
  
  # read the returned data
  TestData = 'R ' + mdi_access_rddata_reg + '\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  return ResponseData[4:18]  
  
def set_reg2():
  c = raw_input('Enter an Error Rate from 1 to 6 => ')
  n = int(c)
  s = '0'
  if n == 1:
    s = '1'
  elif n == 2:
    s = '101'
  elif n == 3:
    s = '10101'
  elif n == 4:
    s = '1010101'
  elif n == 5:
    s = '101010101'
  elif n == 6:
    s = '10101010101'
    
  TestData = 'W ' + test_reg_2_reg + ' ' + s.upper() + '\x0D\x0A'  # assert start_stb
  ser.write(TestData)
  time.sleep(.2)              # delay 
  TestData = 'R ' + test_reg_2_reg + '\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  print 'test_reg_2_reg' + ' = ', ResponseData[4:18]
  if save_log == 1:
    print >> log, 'test_reg_2_reg' + ' = ', ResponseData[4:18]

#Function to limit the number of packet in a test run  
def set_max_packets():         
  global max_tx_packets
  global packet_length
  global mode
  c = raw_input('\nEnter a maximum number of Packets (1-65536) in the Run => ')
  max_tx_packets = 0
  if c > 0:
    max_tx_packets = int(c) - 1
  c = raw_input('\nDo you want to limit the run to n packets? (y/n) => ')
  mode = 0
  if c.upper() == 'Y':
    mode = 1
  n = (max_tx_packets * 4294967296) + packet_length
  s = format(n,'012x')
  TestData = 'W' + ' ' + payload_config_reg + ' ' + s.upper() + '\x0D\x0A'  
  ser.write(TestData)

    
        
# function to start a test run
def start_test():
  global mode
  print '>>> Starting test '
  if save_log == 1:
    print >> log, '>>> Starting test '
  reset_test()
  TestData = 'W ' + test_control_reg + ' ' + '1\x0D\x0A'  # assert start_stb
  if mode == 1:
    TestData = 'W ' + test_control_reg + ' ' + '3\x0D\x0A'  # assert start_stb
  ser.write(TestData)
  
# function to stop a test run
def stop_test():
  TestData = 'W ' + test_control_reg + ' ' + '0\x0D\x0A'  # assert start_stb
  ser.write(TestData)
  time.sleep(.2)              # delay 
  
# function to set the spacing between Tx packets
def set_delay():
  s = raw_input('Enter packet delay in hex => ')
  TestData = 'W ' + test_interval_reg + ' ' + s.upper() + '\x0D\x0A'  # assert start_stb
  ser.write(TestData)
  time.sleep(.2)              # delay 
  TestData = 'R ' + test_interval_reg + '\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  print 'test_interval_reg' + ' = ', ResponseData[4:18]
  if save_log == 1:
    print >> log, 'test_interval_reg' + ' = ', ResponseData[4:18]

# function to print out timestamps of rx error events
def pop_history(m):
  # show the FIFO count register
  TestData = 'R ' + test_status_reg + '\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(23)
#  print 'FIFO Counts  = ', ResponseData[7:23]
  # get the number of timestamps in the FIFO  
  x = int(ResponseData[7:22], 16)
  n = x/16777216
  print n
  if n > 2*m:
    n = 2*m
  elif n == 0:
    n = 2*m
  while n > 0:
    TestData = 'R ' + test_rxerr_fifo_data_reg + '\x0D\x0A'
    ser.write(TestData)
    ResponseData = ser.read(23)
    x = ResponseData[7:22]
    x1 = float(int(x,16))/125000000.0
    TestData = 'R ' + test_rxerr_fifo_data_reg + '\x0D\x0A'
    ser.write(TestData)
    ResponseData = ser.read(23)
    y = ResponseData[7:22]
    y1 = float(int(y,16))/125000000.0
    d = y1-x1
#    print x + '  ' + y
    print x1,y1,d
    if save_log == 1:
#      print >> log, x,y,x1,y1,d
      print >> log, x1,y1,d
    n = n-2
     
    
def print_cmds():
  print '\nCommands: Q S R P D M N Z'  
  
 
# Begin by checking for command line options
#   -p 'USB PORT NAME'  - USB PORT NAME examples 'COM11' , 'USB1'
#   -l  'off'           - 'off' disable writing log file, anything else is enable (default) 
cmd_line_options = get_options(sys.argv)
if cmd_line_options.has_key('-p') == 1:
  uart_port = cmd_line_options['-p']
  uart_port = uart_port
  print 'Using UART PORT: ' + uart_port
  
if cmd_line_options.has_key('-l') == 1:
  if cmd_line_options['-l'].upper() == 'OFF':
    save_log = 0
    print 'Test Logging is Disabled'

else:
  save_log = 1
  print 'Test Logging is Enabled'
      
  

# Start by trying to open the specified UART port
try:
  ser = serial.Serial(port=uart_port)
except:  
  uart_port_is_open = False
  print uart_port + ' is not available'
else:
  uart_port_is_open = True
  ser.baudrate = 921600 
  # select FIVEBITS, SIXBITS, SEVENBITS or EIGHTBITS of data
  #ser.bytesize = serial.FIVEBITS
  #ser.bytesize = serial.SIXBITS
  #ser.bytesize = serial.SEVENBITS
  ser.bytesize = serial.EIGHTBITS
  
  # select STOPBITS_ONE, STOPBIT_TWO or STOPBITS_ONE_POINT_FIVE
  ser.stopbits = serial.STOPBITS_ONE
  #ser.stopbits = serial.STOPBIT_TWO
  #ser.stopbits = serial.STOPBITS_ONE_POINT_FIVE
  
  # select PARITY_NONE, PARITY_EVEN or PARITY_ODD
  ser.parity = serial.PARITY_NONE
  #ser.parity = serial.PARITY_EVEN
  #ser.parity = serial.PARITY_ODD
  
  ser.xonxoff = 0
  ser.rtscts = 0
  # IF you don't set the timeout for reads the code will hang
  ser.timeout = 0.5
  
  #ser.parity = serial.PARITY_EVEN
  
# only continue if the specified UART connection exists  
if uart_port_is_open == True:
    
  # Test R7 to see if we can communicate with the FPGA
  print 'Writing R7 with 0x112233445566 to test interface'
  TestData = 'W ' + test_readback_reg + ' 112233445566\x0D\x0A' # write command followed by CRLF
  ser.write(TestData)
  TestData = 'R ' + test_readback_reg + ' \x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  print 'Reading R0: ',ResponseData
  if ResponseData[:18] == '0x7 0x112233445566':
    print 'Register write/read Test Successful'
    has_connection = True
  else:  
    print 'Register write/read Test Failed!'
    has_connection = False
  
  # After the initial tests we have the main menu
  # but only if we were able to read back R0 successfully
  if has_connection == True:
  
    # open a log file if logging is enabled
    if save_log == 1:
      tstr = time.ctime(time.time())
      tstr1 = tstr.replace(' ', '_')
      tstr1 = tstr.replace(':', '')
      lfname = tstr1 + '_log' + '.txt'
      log = open(lfname, 'a')
      print >> log, tstr
        
  
    u = '~'
    while 1:
      print_cmds()
      u = raw_input('\nEnter Command => ')
      # Quit the test
      if u.upper() == 'Q':
        break
        
      # Start a test                                                         
      elif u.upper() == 'R':                                                                                          
        start_test()
        r = 1
        while r == 1:
          time.sleep(.25)               
#          print str(v)
#          else:  
#          if mode != 0:
          v = read_status()
          if v & test_is_running_mask != 1:
            print '>>> Test Terminated'
            if save_log == 1:
              print >> log, '>>> Test Terminated'
            r = 0
            break
          uu = raw_input()
          if uu.upper() == 'Q':
            print '>>> User Stopped this Test'
            if save_log == 1:
              print >> log, '>>> User Stopped this Test'
            r = 0
          elif uu.upper() == 'Z':   
            show_all()
        stop_test()
        show_all()
        show_results()  
           
      elif u.upper() == 'T':
        v = read_status()
        print v 
        
      elif u.upper() == 'P':
        write_payload_config()  
               
      elif u.upper() == 'D':
        set_delay()  
               
      elif u.upper() == 'S':
        set_reg2()  
               
      elif u.upper() == 'N':
        set_max_packets()  
               
      # Print a PHY register  
      elif u.upper() == 'M':
        c = raw_input('\nEnter a PHY Register Address to Read in Hex => ')
        s0 = int(c,16)
        s = format(s0,'02x')        
        t = read_phy_reg(s)
        print t  
      
      # Print Control/Status registers    
      elif u.upper() == 'Z':
        show_all()
        
      else:
        print '\nNot a valid Command'  
    
  if save_log == 1:
    log.close    
  ser.close

